@@ -25,8 +25,8 @@
 
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "gtk-utils.h"
 #include "fr-window.h"
 
@@ -118,9 +118,9 @@ dlg_ask_password__common (FrWindow       *window,
 	/* Set widgets data. */
 
 	if (data->pwd_type == FR_PASSWORD_TYPE_MAIN)
-		name = gnome_vfs_unescape_string_for_display (file_name_from_path (fr_window_get_archive_uri (window)));
+		name = g_uri_display_basename (fr_window_get_archive_uri (window));
 	else if (data->pwd_type == FR_PASSWORD_TYPE_PASTE_FROM)
-		name = g_filename_display_basename (fr_window_get_paste_archive_uri (window));
+		name = g_uri_display_basename (fr_window_get_paste_archive_uri (window));
 	text = g_strdup_printf (_("Enter the password for the archive '%s'."), name);
 	gtk_label_set_label (GTK_LABEL (label), text);
 	g_free (text);
