@@ -323,9 +323,9 @@ fr_archive_action_completed (FrArchive       *archive,
 	archive->error.status = 0;
 	g_clear_error (&archive->error.gerror);
 	if (error_details != NULL)
-		archive->error.gerror = g_error_new (fr_error_quark (),
-						     0,
-						     error_details);
+		archive->error.gerror = g_error_new_literal (fr_error_quark (),
+							     0,
+							     error_details);
 	g_signal_emit (G_OBJECT (archive),
 		       fr_archive_signals[DONE],
 		       0,
@@ -397,13 +397,13 @@ get_local_copy_for_file (GFile *remote_file)
 	temp_dir = get_temp_work_dir ();
 	if (temp_dir != NULL) {
 		char  *archive_name;
-		char  *local_uri;
+		char  *local_path;
 
 		archive_name = g_file_get_basename (remote_file);
-		local_uri = g_build_filename (temp_dir, archive_name, NULL);
-		local_copy = g_file_new_for_uri (local_uri);
+		local_path = g_build_filename (temp_dir, archive_name, NULL);
+		local_copy = g_file_new_for_path (local_path);
 
-		g_free (local_uri);
+		g_free (local_path);
 		g_free (archive_name);
 	}
 	g_free (temp_dir);
@@ -533,8 +533,10 @@ get_mime_type_from_magic_numbers (GFile *file)
 	} sniffer_data [] = {
 		/* Magic numbers taken from magic/Magdir/archive from the
 		 * file-4.21 tarball. */
+		{ "application/x-7z-compressed", "7z\274\257\047\034", 0, 5 },
 		{ "application/x-ace", "**ACE**", 7, 7 },
 		{ "application/x-arj", "\x60\xea", 0, 2 },
+		{ "application/x-gzip", "\037\213", 0, 2 },
 		{ "application/x-rar", "Rar!", 0, 4 },
 		{ "application/x-rzip", "RZIP", 0, 4 },
 		{ "application/x-zoo", "\xdc\xa7\xc4\xfd", 20, 4 },
@@ -542,17 +544,10 @@ get_mime_type_from_magic_numbers (GFile *file)
 		{ "application/zip", "PK00PK\003\004", 0, 8 },
 		{ NULL, NULL, 0 }
 	};
-	GFileInputStream *istream;
-	char              buffer[32];
-	int               n, i;
+	char buffer[32];
+	int  i;
 
-	istream = g_file_read (file, NULL, NULL);
-	if (istream == NULL)
-		return NULL;
-
-	n = g_input_stream_read (G_INPUT_STREAM (istream), buffer, sizeof (buffer), NULL, NULL);
-	g_object_unref (istream);
-	if (n < 0)
+	if (! g_load_file_in_buffer (file, buffer, 32, NULL))
 		return NULL;
 
 	for (i = 0; sniffer_data[i].mime_type != NULL; i++)
@@ -585,36 +580,13 @@ get_mime_type_from_filename (GFile *file)
 
 
 static gboolean
-create_command_from_mime_type (FrArchive  *archive,
-			       const char *mime_type,
-			       gboolean    loading)
+create_command_from_type (FrArchive     *archive,
+			  const char    *mime_type,
+		          GType          command_type,
+		          FrCommandCaps  requested_capabilities)
 {
-	FrCommandCaps  requested_capabilities = FR_COMMAND_CAN_DO_NOTHING;
-	GType          command_type;
-	char          *filename;
-
-	if (mime_type == NULL)
-		return FALSE;
-
-	archive->is_compressed_file = FALSE;
-
-	/* try with the WRITE capability even when loading, this way we give
-	 * priority to the commands that can read and write over commands
-	 * that can only read a specific file format. */
-
-	requested_capabilities |= FR_COMMAND_CAN_READ_WRITE;
-	if (! loading && ! archive->can_create_compressed_file)
-		requested_capabilities |= FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
-	command_type = get_command_type_from_mime_type (mime_type, requested_capabilities);
-
-	/* if no command was found and we are loading, remove the write
-	 * capability and try again */
-
-	if ((command_type == 0) && loading) {
-		requested_capabilities ^= FR_COMMAND_CAN_WRITE;
-		command_type = get_command_type_from_mime_type (mime_type, requested_capabilities);
-	}
-
+	char *filename;
+	
 	if (command_type == 0)
 		return FALSE;
 
@@ -629,14 +601,66 @@ create_command_from_mime_type (FrArchive  *archive,
 	if (! fr_command_is_capable_of (archive->command, requested_capabilities)) {
 		g_object_unref (archive->command);
 		archive->command = NULL;
+		archive->is_compressed_file = FALSE;
 	}
-	else if (archive->command != NULL)
+	else 
 		archive->is_compressed_file = ! fr_command_is_capable_of (archive->command, FR_COMMAND_CAN_ARCHIVE_MANY_FILES);
 
 	return (archive->command != NULL);
 }
 
 
+static gboolean
+create_command_to_load_archive (FrArchive  *archive,
+			        const char *mime_type)
+{
+	FrCommandCaps requested_capabilities = FR_COMMAND_CAN_DO_NOTHING;
+	GType         command_type;
+
+	if (mime_type == NULL)
+		return FALSE;
+
+	/* try with the WRITE capability even when loading, this way we give
+	 * priority to the commands that can read and write over commands
+	 * that can only read a specific file format. */
+
+	requested_capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	command_type = get_command_type_from_mime_type (mime_type, requested_capabilities);
+
+	/* if no command was found, remove the write capability and try again */
+
+	if (command_type == 0) {
+		requested_capabilities ^= FR_COMMAND_CAN_WRITE;
+		command_type = get_command_type_from_mime_type (mime_type, requested_capabilities);
+	}	
+
+	return create_command_from_type (archive,
+					 mime_type, 
+					 command_type, 
+					 requested_capabilities);
+}
+
+
+static gboolean
+create_command_to_create_archive (FrArchive  *archive,
+			          const char *mime_type)
+{
+	FrCommandCaps requested_capabilities = FR_COMMAND_CAN_DO_NOTHING;
+	GType         command_type;
+
+	if (mime_type == NULL)
+		return FALSE;
+
+	requested_capabilities |= FR_COMMAND_CAN_WRITE;
+	command_type = get_command_type_from_mime_type (mime_type, requested_capabilities);
+
+	return create_command_from_type (archive,
+					 mime_type, 
+					 command_type, 
+					 requested_capabilities);
+}
+
+
 static void
 action_started (FrCommand *command,
 		FrAction   action,
@@ -665,7 +689,7 @@ fr_archive_copy_done (FrArchive *archive,
 	const char      *error_details = NULL;
 
 	if (error != NULL) {
-		error_type = (g_error_matches (error, G_FILE_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC);
+		error_type = (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC);
 		error_details = error->message;
 	}
 	fr_archive_action_completed (archive, action, error_type, error_details);
@@ -856,6 +880,28 @@ copy_extracted_files_to_destination (FrArchive *archive)
 static void add_dropped_items (DroppedItemsData *data);
 
 
+static void
+fr_archive_change_name (FrArchive  *archive,
+		        const char *filename)
+{
+	const char *name;
+	GFile      *parent;
+
+
+	name = file_name_from_path (filename);
+
+	parent = g_file_get_parent (archive->file);
+	g_object_unref (archive->file);
+	archive->file = g_file_get_child (parent, name);
+	g_object_unref (parent);
+
+	parent = g_file_get_parent (archive->local_copy);
+	g_object_unref (archive->local_copy);
+	archive->local_copy = g_file_get_child (parent, name);
+	g_object_unref (parent);
+}
+
+
 static void
 action_performed (FrCommand   *command,
 		  FrAction     action,
@@ -887,6 +933,10 @@ action_performed (FrCommand   *command,
 				dropped_items_data_free (archive->priv->dropped_items_data);
 				archive->priv->dropped_items_data = NULL;
 			}
+			/* the name of the volumes are different from the
+			 * original name */
+			if (archive->command->multi_volume)
+				fr_archive_change_name (archive, archive->command->filename);
 			if (! g_file_has_uri_scheme (archive->file, "file")) {
 				copy_to_remote_location (archive, action);
 				return;
@@ -919,6 +969,10 @@ action_performed (FrCommand   *command,
 		break;
 
 	case FR_ACTION_LISTING_CONTENT:
+		/* the name of the volumes are different from the
+		 * original name */
+		if (archive->command->multi_volume)
+			fr_archive_change_name (archive, archive->command->filename);
 		fr_command_update_capabilities (archive->command);
 		if (! fr_command_is_capable_of (archive->command, FR_COMMAND_CAN_WRITE))
 			archive->read_only = TRUE;
@@ -1017,7 +1071,7 @@ fr_archive_create (FrArchive  *archive,
 	tmp_command = archive->command;
 
 	mime_type = get_mime_type_from_filename (archive->local_copy);
-	if (! create_command_from_mime_type (archive, mime_type, FALSE)) {
+	if (! create_command_to_create_archive (archive, mime_type)) {
 		archive->command = tmp_command;
 		return FALSE;
 	}
@@ -1071,11 +1125,11 @@ load_local_archive (FrArchive  *archive,
 	tmp_command = archive->command;
 
 	mime_type = get_mime_type_from_filename (archive->local_copy);
-	if (! create_command_from_mime_type (archive, mime_type, TRUE)) {
+	if (! create_command_to_load_archive (archive, mime_type)) {
 		mime_type = get_mime_type_from_content (archive->local_copy);
-		if (! create_command_from_mime_type (archive, mime_type, TRUE)) {
+		if (! create_command_to_load_archive (archive, mime_type)) {
 			mime_type = get_mime_type_from_magic_numbers (archive->local_copy);
-			if (! create_command_from_mime_type (archive, mime_type, TRUE)) {
+			if (! create_command_to_load_archive (archive, mime_type)) {
 				archive->command = tmp_command;
 				fr_archive_action_completed (archive,
 							     FR_ACTION_LOADING_ARCHIVE,
@@ -1261,9 +1315,9 @@ fr_archive_rename (FrArchive  *archive,
 		fr_archive_load (archive, filename, NULL);
 	}
 	else {
-		g_object_unref (archive->file);
+		if (archive->file != NULL)
+			g_object_unref (archive->file);
 		archive->file = g_file_new_for_path (filename);
-
 		fr_command_set_filename (archive->command, filename);
 	}
 }
@@ -1285,8 +1339,10 @@ create_tmp_base_dir (const char *base_dir,
 	if ((dest_path == NULL)
 	    || (*dest_path == '\0')
 	    || (strcmp (dest_path, "/") == 0))
+	{
 		return g_strdup (base_dir);
-
+	}
+	
 	dest_dir = g_strdup (dest_path);
 	if (dest_dir[strlen (dest_dir) - 1] == G_DIR_SEPARATOR)
 		dest_dir[strlen (dest_dir) - 1] = 0;
@@ -1454,6 +1510,41 @@ save_list_to_temp_file (GList   *file_list,
 }
 
 
+static GList *
+split_in_chunks (GList *file_list)
+{
+	GList *chunks = NULL;
+	GList *new_file_list;
+	GList *scan;
+
+	new_file_list = g_list_copy (file_list);
+	for (scan = new_file_list; scan != NULL; /* void */) {
+		GList *prev = scan->prev;
+		GList *chunk;
+		int    l;
+
+		chunk = scan;
+		l = 0;
+		while ((scan != NULL) && (l < MAX_CHUNK_LEN)) {
+			if (l == 0)
+				l = strlen (scan->data);
+			prev = scan;
+			scan = scan->next;
+			if (scan != NULL)
+				l += strlen (scan->data);
+		}
+		if (prev != NULL) {
+			if (prev->next != NULL)
+				prev->next->prev = NULL;
+			prev->next = NULL;
+		}
+		chunks = g_list_append (chunks, chunk);
+	}
+
+	return chunks;
+}
+
+
 void
 fr_archive_add (FrArchive     *archive,
 		GList         *file_list,
@@ -1534,6 +1625,8 @@ fr_archive_add (FrArchive     *archive,
 		return;
 	}
 
+	archive->command->creating_archive = ! g_file_test (archive->command->filename, G_FILE_TEST_EXISTS);
+	
 	fr_command_uncompress (archive->command);
 
 	/* when files are already present in a tar archive and are added
@@ -1606,34 +1699,26 @@ fr_archive_add (FrArchive     *archive,
 		g_free (list_dir);
 	}
 	else {
+		GList *chunks = NULL;
+
 		/* specify the file list on the command line, splitting
 		 * in more commands to avoid to overflow the command line
 		 * length limit. */
-		for (scan = new_file_list; scan != NULL; ) {
-			GList *prev = scan->prev;
-			GList *chunk_list;
-			int    l;
 
-			chunk_list = scan;
-			l = 0;
-			while ((scan != NULL) && (l < MAX_CHUNK_LEN)) {
-				if (l == 0)
-					l = strlen (scan->data);
-				prev = scan;
-				scan = scan->next;
-				if (scan != NULL)
-					l += strlen (scan->data);
-			}
+		chunks = split_in_chunks (new_file_list);
+		for (scan = chunks; scan != NULL; scan = scan->next) {
+			GList *chunk = scan->data;
 
-			prev->next = NULL;
 			fr_command_add (archive->command,
 					NULL,
-					chunk_list,
+					chunk,
 					tmp_base_dir,
 					update,
 					recursive);
-			prev->next = scan;
+			g_list_free (chunk);
 		}
+
+		g_list_free (chunks);
 	}
 
 	path_list_free (new_file_list);
@@ -1896,7 +1981,7 @@ add_with_wildcard__step2 (GList    *file_list,
 	if (error != NULL) {
 		fr_archive_action_completed (archive,
 					     FR_ACTION_GETTING_FILE_LIST,
-					     (g_error_matches (error, G_FILE_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC),
+					     (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC),
 					     error->message);
 		return;
 	}
@@ -2009,7 +2094,7 @@ add_directory__step2 (GList    *file_list,
 	if (error != NULL) {
 		fr_archive_action_completed (archive,
 					     FR_ACTION_GETTING_FILE_LIST,
-					     (g_error_matches (error, G_FILE_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC),
+					     (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) ? FR_PROC_ERROR_STOPPED : FR_PROC_ERROR_GENERIC),
 					     error->message);
 		return;
 	}
