@@ -172,16 +172,22 @@ add_password_arg (FrCommand  *comm,
 }
 
 
+static void
+list__begin (gpointer data)
+{
+	FrCommandZip *comm = data;
+	
+	comm->is_empty = FALSE;
+}
+
+
 static void
 fr_command_zip_list (FrCommand  *comm)
 {
-	FR_COMMAND_ZIP (comm)->is_empty = FALSE;
-
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
-				      list__process_line,
-				      comm);
+	fr_process_set_out_line_func (comm->process, list__process_line, comm);
 
 	fr_process_begin_command (comm->process, "unzip");
+	fr_process_set_begin_func (comm->process, list__begin, comm);
 	fr_process_add_arg (comm->process, "-ZTs");
 	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
@@ -357,7 +363,7 @@ fr_command_zip_handle_error (FrCommand   *comm,
 
 
 const char *zip_mime_type[] = { "application/x-cbz",
-				"application/x-executable",
+				"application/x-ms-dos-executable",
 				"application/zip",
 				NULL };
 
@@ -377,7 +383,7 @@ fr_command_zip_get_capabilities (FrCommand  *comm,
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES | FR_COMMAND_CAN_ENCRYPT;
 	if (is_program_in_path ("zip")) {
-		if (strcmp (mime_type, "application/x-executable") == 0)
+		if (strcmp (mime_type, "application/x-ms-dos-executable") == 0)
 			capabilities |= FR_COMMAND_CAN_READ;
 		else
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
